@@ -685,17 +685,15 @@ const WindowIcon = new Lang.Class({
 
         this._icon.destroy_all_children();
 
-        let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
-
         switch (mode) {
             case AppIconMode.THUMBNAIL_ONLY:
                 size = WINDOW_PREVIEW_SIZE;
-                this._icon.add_actor(_createWindowClone(mutterWindow, size * scaleFactor));
+                this._icon.add_actor(_createWindowClone(mutterWindow, WINDOW_PREVIEW_SIZE));
                 break;
 
             case AppIconMode.BOTH:
                 size = WINDOW_PREVIEW_SIZE;
-                this._icon.add_actor(_createWindowClone(mutterWindow, size * scaleFactor));
+                this._icon.add_actor(_createWindowClone(mutterWindow, WINDOW_PREVIEW_SIZE));
 
                 if (this.app)
                     this._icon.add_actor(this._createAppIcon(this.app,
@@ -707,7 +705,7 @@ const WindowIcon = new Lang.Class({
                 this._icon.add_actor(this._createAppIcon(this.app, size));
         }
 
-        this._icon.set_size(size * scaleFactor, size * scaleFactor);
+        this._icon.set_size(size, size);
     },
 
     _createAppIcon: function(app, size) {
