@@ -211,7 +211,7 @@ calculate_gaussian_kernel (gdouble   sigma,
 {
   gdouble *ret, sum;
   gdouble exp_divisor;
-  int half, i;
+  guint half, i;
 
   g_return_val_if_fail (sigma > 0, NULL);
 
@@ -223,14 +223,14 @@ calculate_gaussian_kernel (gdouble   sigma,
   exp_divisor = 2 * sigma * sigma;
 
   /* n_values of 1D Gauss function */
-  for (i = 0; i < (int)n_values; i++)
+  for (i = 0; i < n_values; i++)
     {
       ret[i] = exp (-(i - half) * (i - half) / exp_divisor);
       sum += ret[i];
     }
 
   /* normalize */
-  for (i = 0; i < (int)n_values; i++)
+  for (i = 0; i < n_values; i++)
     ret[i] /= sum;
 
   return ret;
@@ -347,7 +347,6 @@ _st_create_shadow_pipeline (StShadow    *shadow_spec,
 {
   ClutterBackend *backend = clutter_get_default_backend ();
   CoglContext *ctx = clutter_backend_get_cogl_context (backend);
-  CoglError *error = NULL;
 
   static CoglPipeline *shadow_pipeline_template = NULL;
 
@@ -378,13 +377,7 @@ _st_create_shadow_pipeline (StShadow    *shadow_spec,
                                                          COGL_PIXEL_FORMAT_A_8,
                                                          rowstride_out,
                                                          pixels_out,
-                                                         &error));
-
-  if (error)
-    {
-      g_warning ("Failed to allocate texture: %s", error->message);
-      cogl_error_free (error);
-    }
+                                                         NULL));
 
   g_free (pixels_out);
 
@@ -402,10 +395,7 @@ _st_create_shadow_pipeline (StShadow    *shadow_spec,
 
   pipeline = cogl_pipeline_copy (shadow_pipeline_template);
   cogl_pipeline_set_layer_texture (pipeline, 0, texture);
-
-  if (texture)
-    cogl_object_unref (texture);
-
+  cogl_object_unref (texture);
   return pipeline;
 }
 
@@ -452,7 +442,6 @@ _st_create_shadow_pipeline_from_actor (StShadow     *shadow_spec,
       if (!cogl_framebuffer_allocate (fb, &catch_error))
         {
           cogl_error_free (catch_error);
-          cogl_object_unref (offscreen);
           cogl_object_unref (buffer);
           return NULL;
         }
