@@ -71,7 +71,6 @@ create_corner_material (StCornerSpec *corner)
 {
   ClutterBackend *backend = clutter_get_default_backend ();
   CoglContext *ctx = clutter_backend_get_cogl_context (backend);
-  CoglError *error = NULL;
   CoglHandle texture;
   cairo_t *cr;
   cairo_surface_t *surface;
@@ -173,15 +172,9 @@ create_corner_material (StCornerSpec *corner)
                                                          CLUTTER_CAIRO_FORMAT_ARGB32,
                                                          rowstride,
                                                          data,
-                                                         &error));
-
-  if (error)
-    {
-      g_warning ("Failed to allocate texture: %s", error->message);
-      cogl_error_free (error);
-    }
-
+                                                         NULL));
   g_free (data);
+  g_assert (texture != COGL_INVALID_HANDLE);
 
   return texture;
 }
@@ -354,7 +347,7 @@ st_theme_node_lookup_corner (StThemeNode    *node,
                              float           height,
                              StCorner        corner_id)
 {
-  CoglHandle texture, material = COGL_INVALID_HANDLE;
+  CoglHandle texture, material;
   char *key;
   StTextureCache *cache;
   StCornerSpec corner;
@@ -403,12 +396,8 @@ st_theme_node_lookup_corner (StThemeNode    *node,
 
   key = corner_to_string (&corner);
   texture = st_texture_cache_load (cache, key, ST_TEXTURE_CACHE_POLICY_NONE, load_corner, &corner, NULL);
-
-  if (texture)
-    {
-      material = _st_create_texture_pipeline (texture);
-      cogl_handle_unref (texture);
-    }
+  material = _st_create_texture_pipeline (texture);
+  cogl_handle_unref (texture);
 
   g_free (key);
 
@@ -969,7 +958,6 @@ st_theme_node_prerender_background (StThemeNode *node,
 {
   ClutterBackend *backend = clutter_get_default_backend ();
   CoglContext *ctx = clutter_backend_get_cogl_context (backend);
-  CoglError *error = NULL;
   StBorderImage *border_image;
   CoglHandle texture;
   guint radius[4];
@@ -1289,13 +1277,7 @@ st_theme_node_prerender_background (StThemeNode *node,
                                                          CLUTTER_CAIRO_FORMAT_ARGB32,
                                                          rowstride,
                                                          data,
-                                                         &error));
-
-  if (error)
-    {
-      g_warning ("Failed to allocate texture: %s", error->message);
-      cogl_error_free (error);
-    }
+                                                         NULL));
 
   cairo_destroy (cr);
   cairo_surface_destroy (surface);
@@ -1305,6 +1287,7 @@ st_theme_node_prerender_background (StThemeNode *node,
 }
 
 static void st_theme_node_paint_borders (StThemeNodePaintState *state,
+                                         CoglFramebuffer       *framebuffer,
                                          const ClutterActorBox *box,
                                          guint8                 paint_opacity);
 
@@ -1317,10 +1300,10 @@ st_theme_node_invalidate_border_image (StThemeNode *node)
       node->border_slices_texture = COGL_INVALID_HANDLE;
     }
 
-  if (node->border_slices_material != COGL_INVALID_HANDLE)
+  if (node->border_slices_pipeline != COGL_INVALID_HANDLE)
     {
-      cogl_handle_unref (node->border_slices_material);
-      node->border_slices_material = COGL_INVALID_HANDLE;
+      cogl_handle_unref (node->border_slices_pipeline);
+      node->border_slices_pipeline = COGL_INVALID_HANDLE;
     }
 }
 
@@ -1346,7 +1329,7 @@ st_theme_node_load_border_image (StThemeNode *node)
       if (node->border_slices_texture == COGL_INVALID_HANDLE)
         goto out;
 
-      node->border_slices_material = _st_create_texture_pipeline (node->border_slices_texture);
+      node->border_slices_pipeline = _st_create_texture_pipeline (node->border_slices_texture);
     }
 
  out:
@@ -1362,16 +1345,16 @@ st_theme_node_invalidate_background_image (StThemeNode *node)
       node->background_texture = COGL_INVALID_HANDLE;
     }
 
-  if (node->background_material != COGL_INVALID_HANDLE)
+  if (node->background_pipeline != COGL_INVALID_HANDLE)
     {
-      cogl_handle_unref (node->background_material);
-      node->background_material = COGL_INVALID_HANDLE;
+      cogl_handle_unref (node->background_pipeline);
+      node->background_pipeline = COGL_INVALID_HANDLE;
     }
 
-  if (node->background_shadow_material != COGL_INVALID_HANDLE)
+  if (node->background_shadow_pipeline != COGL_INVALID_HANDLE)
     {
-      cogl_handle_unref (node->background_shadow_material);
-      node->background_shadow_material = COGL_INVALID_HANDLE;
+      cogl_handle_unref (node->background_shadow_pipeline);
+      node->background_shadow_pipeline = COGL_INVALID_HANDLE;
     }
 }
 
@@ -1396,14 +1379,15 @@ st_theme_node_load_background_image (StThemeNode *node)
       if (node->background_texture == COGL_INVALID_HANDLE)
         goto out;
 
-      node->background_material = _st_create_texture_pipeline (node->background_texture);
+      node->background_pipeline = _st_create_texture_pipeline (node->background_texture);
 
       if (node->background_repeat)
-        cogl_material_set_layer_wrap_mode (node->background_material, 0, COGL_MATERIAL_WRAP_MODE_REPEAT);
+        cogl_pipeline_set_layer_wrap_mode (node->background_pipeline, 0,
+                                           COGL_PIPELINE_WRAP_MODE_REPEAT);
 
       if (background_image_shadow_spec)
         {
-          node->background_shadow_material = _st_create_shadow_pipeline (background_image_shadow_spec,
+          node->background_shadow_pipeline = _st_create_shadow_pipeline (background_image_shadow_spec,
                                                                          node->background_texture);
         }
     }
@@ -1506,17 +1490,17 @@ st_theme_node_render_resources (StThemeNodePaintState *state,
     state->prerendered_texture = st_theme_node_prerender_background (node, width, height);
 
   if (state->prerendered_texture)
-    state->prerendered_material = _st_create_texture_pipeline (state->prerendered_texture);
+    state->prerendered_pipeline = _st_create_texture_pipeline (state->prerendered_texture);
   else
-    state->prerendered_material = COGL_INVALID_HANDLE;
+    state->prerendered_pipeline = COGL_INVALID_HANDLE;
 
   if (box_shadow_spec && !has_inset_box_shadow)
     {
       if (st_theme_node_load_border_image (node))
-        state->box_shadow_material = _st_create_shadow_pipeline (box_shadow_spec,
+        state->box_shadow_pipeline = _st_create_shadow_pipeline (box_shadow_spec,
                                                                  node->border_slices_texture);
       else if (state->prerendered_texture != COGL_INVALID_HANDLE)
-        state->box_shadow_material = _st_create_shadow_pipeline (box_shadow_spec,
+        state->box_shadow_pipeline = _st_create_shadow_pipeline (box_shadow_spec,
                                                                  state->prerendered_texture);
       else if (node->background_color.alpha > 0 || has_border)
         st_theme_node_prerender_shadow (state);
@@ -1526,7 +1510,7 @@ st_theme_node_render_resources (StThemeNodePaintState *state,
      them. */
   if (!node->cached_textures)
     {
-      if (state->prerendered_material == COGL_INVALID_HANDLE &&
+      if (state->prerendered_pipeline == COGL_INVALID_HANDLE &&
           width >= node->box_shadow_min_width &&
           height >= node->box_shadow_min_height)
         {
@@ -1555,16 +1539,16 @@ st_theme_node_update_resources (StThemeNodePaintState *state,
       state->prerendered_texture = COGL_INVALID_HANDLE;
       had_prerendered_texture = TRUE;
     }
-  if (state->prerendered_material != COGL_INVALID_HANDLE)
+  if (state->prerendered_pipeline != COGL_INVALID_HANDLE)
     {
-      cogl_handle_unref (state->prerendered_material);
-      state->prerendered_material = COGL_INVALID_HANDLE;
+      cogl_handle_unref (state->prerendered_pipeline);
+      state->prerendered_pipeline = COGL_INVALID_HANDLE;
 
       if (node->border_slices_texture == COGL_INVALID_HANDLE &&
-          state->box_shadow_material != COGL_INVALID_HANDLE)
+          state->box_shadow_pipeline != COGL_INVALID_HANDLE)
         {
-          cogl_handle_unref (state->box_shadow_material);
-          state->box_shadow_material = COGL_INVALID_HANDLE;
+          cogl_handle_unref (state->box_shadow_pipeline);
+          state->box_shadow_pipeline = COGL_INVALID_HANDLE;
           had_box_shadow = TRUE;
         }
     }
@@ -1578,7 +1562,7 @@ st_theme_node_update_resources (StThemeNodePaintState *state,
   if (had_prerendered_texture)
     {
       state->prerendered_texture = st_theme_node_prerender_background (node, width, height);
-      state->prerendered_material = _st_create_texture_pipeline (state->prerendered_texture);
+      state->prerendered_pipeline = _st_create_texture_pipeline (state->prerendered_texture);
     }
   else
     {
@@ -1591,7 +1575,7 @@ st_theme_node_update_resources (StThemeNodePaintState *state,
     }
 
   if (had_box_shadow)
-    state->box_shadow_material = _st_create_shadow_pipeline (box_shadow_spec,
+    state->box_shadow_pipeline = _st_create_shadow_pipeline (box_shadow_spec,
                                                              state->prerendered_texture);
 }
 
@@ -1601,20 +1585,23 @@ paint_material_with_opacity (CoglHandle       material,
                              ClutterActorBox *coords,
                              guint8           paint_opacity)
 {
-  cogl_material_set_color4ub (material,
+  CoglFramebuffer *fb = cogl_get_draw_framebuffer ();
+
+  cogl_pipeline_set_color4ub (material,
                               paint_opacity, paint_opacity, paint_opacity, paint_opacity);
 
-  cogl_set_source (material);
-
   if (coords)
-    cogl_rectangle_with_texture_coords (box->x1, box->y1, box->x2, box->y2,
-                                        coords->x1, coords->y1, coords->x2, coords->y2);
+    cogl_framebuffer_draw_textured_rectangle (fb, material,
+                                              box->x1, box->y1, box->x2, box->y2,
+                                              coords->x1, coords->y1, coords->x2, coords->y2);
   else
-    cogl_rectangle (box->x1, box->y1, box->x2, box->y2);
+    cogl_framebuffer_draw_rectangle (fb, material,
+                                     box->x1, box->y1, box->x2, box->y2);
 }
 
 static void
 st_theme_node_paint_borders (StThemeNodePaintState *state,
+                             CoglFramebuffer       *framebuffer,
                              const ClutterActorBox *box,
                              guint8                 paint_opacity)
 {
@@ -1725,32 +1712,39 @@ st_theme_node_paint_borders (StThemeNodePaintState *state,
           if (state->corner_material[corner_id] == COGL_INVALID_HANDLE)
             continue;
 
-          cogl_material_set_color4ub (state->corner_material[corner_id],
+          cogl_pipeline_set_color4ub (state->corner_material[corner_id],
                                       paint_opacity, paint_opacity,
                                       paint_opacity, paint_opacity);
-          cogl_set_source (state->corner_material[corner_id]);
 
           switch (corner_id)
             {
               case ST_CORNER_TOPLEFT:
-                cogl_rectangle_with_texture_coords (0, 0,
-                                                    max_width_radius[ST_CORNER_TOPLEFT], max_width_radius[ST_CORNER_TOPLEFT],
-                                                    0, 0, 0.5, 0.5);
+                cogl_framebuffer_draw_textured_rectangle (framebuffer,
+                  state->corner_material[corner_id], 0, 0,
+                  max_width_radius[ST_CORNER_TOPLEFT], max_width_radius[ST_CORNER_TOPLEFT],
+                  0, 0, 0.5, 0.5);
                 break;
               case ST_CORNER_TOPRIGHT:
-                cogl_rectangle_with_texture_coords (width - max_width_radius[ST_CORNER_TOPRIGHT], 0,
-                                                    width, max_width_radius[ST_CORNER_TOPRIGHT],
-                                                    0.5, 0, 1, 0.5);
+                cogl_framebuffer_draw_textured_rectangle (framebuffer,
+                  state->corner_material[corner_id],
+                  width - max_width_radius[ST_CORNER_TOPRIGHT], 0,
+                  width, max_width_radius[ST_CORNER_TOPRIGHT],
+                  0.5, 0, 1, 0.5);
                 break;
               case ST_CORNER_BOTTOMRIGHT:
-                cogl_rectangle_with_texture_coords (width - max_width_radius[ST_CORNER_BOTTOMRIGHT], height - max_width_radius[ST_CORNER_BOTTOMRIGHT],
-                                                    width, height,
-                                                    0.5, 0.5, 1, 1);
+                cogl_framebuffer_draw_textured_rectangle (framebuffer,
+                  state->corner_material[corner_id],
+                  width - max_width_radius[ST_CORNER_BOTTOMRIGHT],
+                  height - max_width_radius[ST_CORNER_BOTTOMRIGHT],
+                  width, height,
+                  0.5, 0.5, 1, 1);
                 break;
               case ST_CORNER_BOTTOMLEFT:
-                cogl_rectangle_with_texture_coords (0, height - max_width_radius[ST_CORNER_BOTTOMLEFT],
-                                                    max_width_radius[ST_CORNER_BOTTOMLEFT], height,
-                                                    0, 0.5, 0.5, 1);
+                cogl_framebuffer_draw_textured_rectangle (framebuffer,
+                  state->corner_material[corner_id],
+                  0, height - max_width_radius[ST_CORNER_BOTTOMLEFT],
+                  max_width_radius[ST_CORNER_BOTTOMLEFT], height,
+                  0, 0.5, 0.5, 1);
                 break;
               default:
                 g_assert_not_reached();
@@ -1891,6 +1885,7 @@ st_theme_node_paint_sliced_shadow (StThemeNodePaintState *state,
                                    guint8                 paint_opacity)
 {
   StThemeNode *node = state->node;
+  CoglFramebuffer *fb = cogl_get_draw_framebuffer ();
   guint border_radius[4];
   CoglColor color;
   StShadow *box_shadow_spec;
@@ -1992,16 +1987,14 @@ st_theme_node_paint_sliced_shadow (StThemeNodePaintState *state,
   right += xoffset;
 
   /* Setup pipeline */
-  cogl_color_set_from_4ub (&color,
-                           box_shadow_spec->color.red   * paint_opacity / 255,
-                           box_shadow_spec->color.green * paint_opacity / 255,
-                           box_shadow_spec->color.blue  * paint_opacity / 255,
-                           box_shadow_spec->color.alpha * paint_opacity / 255);
+  cogl_color_init_from_4ub (&color,
+                            box_shadow_spec->color.red   * paint_opacity / 255,
+                            box_shadow_spec->color.green * paint_opacity / 255,
+                            box_shadow_spec->color.blue  * paint_opacity / 255,
+                            box_shadow_spec->color.alpha * paint_opacity / 255);
   cogl_color_premultiply (&color);
 
-  cogl_material_set_layer_combine_constant (state->box_shadow_material, 0, &color);
-
-  cogl_set_source (state->box_shadow_material);
+  cogl_pipeline_set_layer_combine_constant (state->box_shadow_pipeline, 0, &color);
 
   idx = 0;
 
@@ -2129,7 +2122,8 @@ st_theme_node_paint_sliced_shadow (StThemeNodePaintState *state,
         }
     }
 
-  cogl_rectangles_with_texture_coords (rectangles, idx / 8);
+  cogl_framebuffer_draw_textured_rectangles (fb, state->box_shadow_pipeline,
+                                             rectangles, idx / 8);
 
 #if 0
   /* Visual feedback on shadow's 9-slice and orignal offscreen buffer,
@@ -2210,24 +2204,21 @@ st_theme_node_prerender_shadow (StThemeNodePaintState *state)
                                        COGL_TEXTURE_NO_SLICING,
                                        COGL_PIXEL_FORMAT_ANY);
   if (buffer != COGL_INVALID_HANDLE)
-    offscreen = cogl_offscreen_new_to_texture (buffer);
+    offscreen = cogl_offscreen_new_with_texture (buffer);
 
   if (offscreen != COGL_INVALID_HANDLE)
     {
       ClutterActorBox box = { 0, 0, state->box_shadow_width, state->box_shadow_height};
-      CoglColor clear_color;
 
-      cogl_push_framebuffer (offscreen);
-      cogl_ortho (0, state->box_shadow_width, state->box_shadow_height, 0, 0, 1.0);
+      cogl_framebuffer_orthographic (offscreen, 0, 0,
+                                     state->box_shadow_width,
+                                     state->box_shadow_height, 0, 1.0);
+      cogl_framebuffer_clear4f (offscreen, COGL_BUFFER_BIT_COLOR, 0, 0, 0, 0);
 
-      cogl_color_set_from_4ub (&clear_color, 0, 0, 0, 0);
-      cogl_clear (&clear_color, COGL_BUFFER_BIT_COLOR);
-
-      st_theme_node_paint_borders (state, &box, 0xFF);
-      cogl_pop_framebuffer ();
+      st_theme_node_paint_borders (state, offscreen, &box, 0xFF);
       cogl_handle_unref (offscreen);
 
-      state->box_shadow_material = _st_create_shadow_pipeline (st_theme_node_get_box_shadow (node),
+      state->box_shadow_pipeline = _st_create_shadow_pipeline (st_theme_node_get_box_shadow (node),
                                                                buffer);
     }
 
@@ -2241,12 +2232,13 @@ st_theme_node_paint_sliced_border_image (StThemeNode           *node,
                                          float                  height,
                                          guint8                 paint_opacity)
 {
+  CoglFramebuffer *fb = cogl_get_draw_framebuffer ();
   gfloat ex, ey;
   gfloat tx1, ty1, tx2, ty2;
   gint border_left, border_right, border_top, border_bottom;
   float img_width, img_height;
   StBorderImage *border_image;
-  CoglHandle material;
+  CoglHandle pipeline;
 
   border_image = st_theme_node_get_border_image (node);
   g_assert (border_image != NULL);
@@ -2270,12 +2262,10 @@ st_theme_node_paint_sliced_border_image (StThemeNode           *node,
   if (ey < 0)
     ey = border_bottom;          /* FIXME ? */
 
-  material = node->border_slices_material;
-  cogl_material_set_color4ub (material,
+  pipeline = node->border_slices_pipeline;
+  cogl_pipeline_set_color4ub (pipeline,
                               paint_opacity, paint_opacity, paint_opacity, paint_opacity);
 
-  cogl_set_source (material);
-
   {
     float rectangles[] =
     {
@@ -2325,7 +2315,7 @@ st_theme_node_paint_sliced_border_image (StThemeNode           *node,
       1.0, 1.0
     };
 
-    cogl_rectangles_with_texture_coords (rectangles, 9);
+    cogl_framebuffer_draw_textured_rectangles (fb, pipeline, rectangles, 9);
   }
 }
 
@@ -2428,6 +2418,7 @@ st_theme_node_needs_new_box_shadow_for_size (StThemeNodePaintState *state,
 void
 st_theme_node_paint (StThemeNode           *node,
                      StThemeNodePaintState *state,
+                     CoglFramebuffer       *framebuffer,
                      const ClutterActorBox *box,
                      guint8                 paint_opacity)
 {
@@ -2493,12 +2484,12 @@ st_theme_node_paint (StThemeNode           *node,
    *    such that it's aligned to the outside edges)
    */
 
-  if (state->box_shadow_material)
+  if (state->box_shadow_pipeline)
     {
       if (state->alloc_width < node->box_shadow_min_width ||
           state->alloc_height < node->box_shadow_min_height)
         _st_paint_shadow_with_opacity (node->box_shadow,
-                                       state->box_shadow_material,
+                                       state->box_shadow_pipeline,
                                        &allocation,
                                        paint_opacity);
       else
@@ -2507,10 +2498,10 @@ st_theme_node_paint (StThemeNode           *node,
                                            paint_opacity);
     }
 
-  if (state->prerendered_material != COGL_INVALID_HANDLE ||
+  if (state->prerendered_pipeline != COGL_INVALID_HANDLE ||
       st_theme_node_load_border_image (node))
     {
-      if (state->prerendered_material != COGL_INVALID_HANDLE)
+      if (state->prerendered_pipeline != COGL_INVALID_HANDLE)
         {
           ClutterActorBox paint_box;
 
@@ -2518,23 +2509,23 @@ st_theme_node_paint (StThemeNode           *node,
                                                   &allocation,
                                                   &paint_box);
 
-          paint_material_with_opacity (state->prerendered_material,
+          paint_material_with_opacity (state->prerendered_pipeline,
                                        &paint_box,
                                        NULL,
                                        paint_opacity);
         }
 
-      if (node->border_slices_material != COGL_INVALID_HANDLE)
+      if (node->border_slices_pipeline != COGL_INVALID_HANDLE)
         st_theme_node_paint_sliced_border_image (node, width, height, paint_opacity);
     }
   else
     {
-      st_theme_node_paint_borders (state, box, paint_opacity);
+      st_theme_node_paint_borders (state, framebuffer, box, paint_opacity);
     }
 
   st_theme_node_paint_outline (node, box, paint_opacity);
 
-  if (state->prerendered_material == COGL_INVALID_HANDLE &&
+  if (state->prerendered_pipeline == COGL_INVALID_HANDLE &&
       st_theme_node_load_background_image (node))
     {
       ClutterActorBox background_box;
@@ -2550,7 +2541,9 @@ st_theme_node_paint (StThemeNode           *node,
       get_background_position (node, &allocation, &background_box, &texture_coords);
 
       if (has_visible_outline || node->background_repeat)
-        cogl_clip_push_rectangle (allocation.x1, allocation.y1, allocation.x2, allocation.y2);
+        cogl_framebuffer_push_rectangle_clip (framebuffer,
+                                              allocation.x1, allocation.y1,
+                                              allocation.x2, allocation.y2);
 
       /* CSS based drop shadows
        *
@@ -2565,19 +2558,19 @@ st_theme_node_paint (StThemeNode           *node,
        * there is nothing (like a border, or the edge of the background color)
        * to logically confine it.
        */
-      if (node->background_shadow_material != COGL_INVALID_HANDLE)
+      if (node->background_shadow_pipeline != COGL_INVALID_HANDLE)
         _st_paint_shadow_with_opacity (node->background_image_shadow,
-                                       node->background_shadow_material,
+                                       node->background_shadow_pipeline,
                                        &background_box,
                                        paint_opacity);
 
-      paint_material_with_opacity (node->background_material,
+      paint_material_with_opacity (node->background_pipeline,
                                    &background_box,
                                    &texture_coords,
                                    paint_opacity);
 
       if (has_visible_outline || node->background_repeat)
-        cogl_clip_pop ();
+        cogl_framebuffer_pop_clip (framebuffer);
     }
 }
 
@@ -2589,10 +2582,10 @@ st_theme_node_paint_state_node_free_internal (StThemeNodePaintState *state,
 
   if (state->prerendered_texture != COGL_INVALID_HANDLE)
     cogl_handle_unref (state->prerendered_texture);
-  if (state->prerendered_material != COGL_INVALID_HANDLE)
-    cogl_handle_unref (state->prerendered_material);
-  if (state->box_shadow_material != COGL_INVALID_HANDLE)
-    cogl_handle_unref (state->box_shadow_material);
+  if (state->prerendered_pipeline != COGL_INVALID_HANDLE)
+    cogl_handle_unref (state->prerendered_pipeline);
+  if (state->box_shadow_pipeline != COGL_INVALID_HANDLE)
+    cogl_handle_unref (state->box_shadow_pipeline);
 
   for (corner_id = 0; corner_id < 4; corner_id++)
     if (state->corner_material[corner_id] != COGL_INVALID_HANDLE)
@@ -2639,9 +2632,9 @@ st_theme_node_paint_state_init (StThemeNodePaintState *state)
   state->alloc_width = 0;
   state->alloc_height = 0;
   state->node = NULL;
-  state->box_shadow_material = COGL_INVALID_HANDLE;
+  state->box_shadow_pipeline = COGL_INVALID_HANDLE;
   state->prerendered_texture = COGL_INVALID_HANDLE;
-  state->prerendered_material = COGL_INVALID_HANDLE;
+  state->prerendered_pipeline = COGL_INVALID_HANDLE;
 
   for (corner_id = 0; corner_id < 4; corner_id++)
     state->corner_material[corner_id] = COGL_INVALID_HANDLE;
@@ -2665,12 +2658,12 @@ st_theme_node_paint_state_copy (StThemeNodePaintState *state,
   state->box_shadow_width = other->box_shadow_width;
   state->box_shadow_height = other->box_shadow_height;
 
-  if (other->box_shadow_material)
-    state->box_shadow_material = cogl_handle_ref (other->box_shadow_material);
+  if (other->box_shadow_pipeline)
+    state->box_shadow_pipeline = cogl_handle_ref (other->box_shadow_pipeline);
   if (other->prerendered_texture)
     state->prerendered_texture = cogl_handle_ref (other->prerendered_texture);
-  if (other->prerendered_material)
-    state->prerendered_material = cogl_handle_ref (other->prerendered_material);
+  if (other->prerendered_pipeline)
+    state->prerendered_pipeline = cogl_handle_ref (other->prerendered_pipeline);
   for (corner_id = 0; corner_id < 4; corner_id++)
     if (other->corner_material[corner_id])
       state->corner_material[corner_id] = cogl_handle_ref (other->corner_material[corner_id]);
