@@ -27,7 +27,13 @@ enum {
   LAST_SIGNAL
 };
 
-#define ST_THEME_NODE_TRANSITION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), ST_TYPE_THEME_NODE_TRANSITION, StThemeNodeTransitionPrivate))
+typedef struct _StThemeNodeTransitionPrivate StThemeNodeTransitionPrivate;
+
+struct _StThemeNodeTransition {
+  GObject parent;
+
+  StThemeNodeTransitionPrivate *priv;
+};
 
 struct _StThemeNodeTransitionPrivate {
   StThemeNode *old_theme_node;
@@ -57,7 +63,7 @@ struct _StThemeNodeTransitionPrivate {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (StThemeNodeTransition, st_theme_node_transition, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (StThemeNodeTransition, st_theme_node_transition, G_TYPE_OBJECT);
 
 
 static void
@@ -82,14 +88,12 @@ st_theme_node_transition_new (StThemeNode *from_node,
                               guint        duration)
 {
   StThemeNodeTransition *transition;
-
   g_return_val_if_fail (ST_IS_THEME_NODE (from_node), NULL);
   g_return_val_if_fail (ST_IS_THEME_NODE (to_node), NULL);
 
   duration = st_theme_node_get_transition_duration (to_node);
 
-  transition = g_object_new (ST_TYPE_THEME_NODE_TRANSITION,
-                             NULL);
+  transition = g_object_new (ST_TYPE_THEME_NODE_TRANSITION, NULL);
 
   transition->priv->old_theme_node = g_object_ref (from_node);
   transition->priv->new_theme_node = g_object_ref (to_node);
@@ -123,13 +127,14 @@ void
 st_theme_node_transition_update (StThemeNodeTransition *transition,
                                  StThemeNode           *new_node)
 {
-  StThemeNodeTransitionPrivate *priv = transition->priv;
+  StThemeNodeTransitionPrivate *priv;
   StThemeNode *old_node;
   ClutterTimelineDirection direction;
 
   g_return_if_fail (ST_IS_THEME_NODE_TRANSITION (transition));
   g_return_if_fail (ST_IS_THEME_NODE (new_node));
 
+  priv = transition->priv;
   direction = clutter_timeline_get_direction (priv->timeline);
   old_node = (direction == CLUTTER_TIMELINE_FORWARD) ? priv->old_theme_node
                                                      : priv->new_theme_node;
@@ -235,7 +240,6 @@ setup_framebuffers (StThemeNodeTransition *transition,
                     const ClutterActorBox *allocation)
 {
   StThemeNodeTransitionPrivate *priv = transition->priv;
-  CoglColor clear_color = { 0, 0, 0, 0 };
   guint width, height;
 
   /* template material to avoid unnecessary shader compilation */
@@ -264,11 +268,11 @@ setup_framebuffers (StThemeNodeTransition *transition,
 
   if (priv->old_offscreen)
     cogl_handle_unref (priv->old_offscreen);
-  priv->old_offscreen = cogl_offscreen_new_to_texture (priv->old_texture);
+  priv->old_offscreen = cogl_offscreen_new_with_texture (priv->old_texture);
 
   if (priv->new_offscreen)
     cogl_handle_unref (priv->new_offscreen);
-  priv->new_offscreen = cogl_offscreen_new_to_texture (priv->new_texture);
+  priv->new_offscreen = cogl_offscreen_new_with_texture (priv->new_texture);
 
   g_return_val_if_fail (priv->old_offscreen != COGL_INVALID_HANDLE, FALSE);
   g_return_val_if_fail (priv->new_offscreen != COGL_INVALID_HANDLE, FALSE);
@@ -277,42 +281,49 @@ setup_framebuffers (StThemeNodeTransition *transition,
     {
       if (G_UNLIKELY (material_template == COGL_INVALID_HANDLE))
         {
-          material_template = cogl_material_new ();
+          CoglContext *ctx =
+            clutter_backend_get_cogl_context (clutter_get_default_backend ());
+          material_template = cogl_pipeline_new (ctx);
 
-          cogl_material_set_layer_combine (material_template, 0,
+          cogl_pipeline_set_layer_combine (material_template, 0,
                                            "RGBA = REPLACE (TEXTURE)",
                                            NULL);
-          cogl_material_set_layer_combine (material_template, 1,
+          cogl_pipeline_set_layer_combine (material_template, 1,
                                            "RGBA = INTERPOLATE (PREVIOUS, "
                                                                "TEXTURE, "
                                                                "CONSTANT[A])",
                                            NULL);
-          cogl_material_set_layer_combine (material_template, 2,
+          cogl_pipeline_set_layer_combine (material_template, 2,
                                            "RGBA = MODULATE (PREVIOUS, "
                                                             "PRIMARY)",
                                            NULL);
         }
-      priv->material = cogl_material_copy (material_template);
+      priv->material = cogl_pipeline_copy (material_template);
     }
 
-  cogl_material_set_layer (priv->material, 0, priv->new_texture);
-  cogl_material_set_layer (priv->material, 1, priv->old_texture);
+  cogl_pipeline_set_layer_texture (priv->material, 0, priv->new_texture);
+  cogl_pipeline_set_layer_texture (priv->material, 1, priv->old_texture);
 
-  cogl_push_framebuffer (priv->old_offscreen);
-  cogl_clear (&clear_color, COGL_BUFFER_BIT_COLOR);
-  cogl_ortho (priv->offscreen_box.x1, priv->offscreen_box.x2,
-              priv->offscreen_box.y2, priv->offscreen_box.y1,
-              0.0, 1.0);
-  st_theme_node_paint (priv->old_theme_node, &priv->old_paint_state, allocation, 255);
-  cogl_pop_framebuffer ();
+  cogl_framebuffer_clear4f (priv->old_offscreen, COGL_BUFFER_BIT_COLOR,
+                            0, 0, 0, 0);
+  cogl_framebuffer_orthographic (priv->old_offscreen,
+                                 priv->offscreen_box.x1,
+                                 priv->offscreen_box.y1,
+                                 priv->offscreen_box.x2,
+                                 priv->offscreen_box.y2, 0.0, 1.0);
 
-  cogl_push_framebuffer (priv->new_offscreen);
-  cogl_clear (&clear_color, COGL_BUFFER_BIT_COLOR);
-  cogl_ortho (priv->offscreen_box.x1, priv->offscreen_box.x2,
-              priv->offscreen_box.y2, priv->offscreen_box.y1,
-              0.0, 1.0);
-  st_theme_node_paint (priv->new_theme_node, &priv->new_paint_state, allocation, 255);
-  cogl_pop_framebuffer ();
+  st_theme_node_paint (priv->old_theme_node, &priv->old_paint_state,
+                       priv->old_offscreen, allocation, 255);
+
+  cogl_framebuffer_clear4f (priv->new_offscreen, COGL_BUFFER_BIT_COLOR,
+                            0, 0, 0, 0);
+  cogl_framebuffer_orthographic (priv->new_offscreen,
+                                 priv->offscreen_box.x1,
+                                 priv->offscreen_box.y1,
+                                 priv->offscreen_box.x2,
+                                 priv->offscreen_box.y2, 0.0, 1.0);
+  st_theme_node_paint (priv->new_theme_node, &priv->new_paint_state,
+                       priv->new_offscreen, allocation, 255);
 
   return TRUE;
 }
@@ -323,6 +334,7 @@ st_theme_node_transition_paint (StThemeNodeTransition *transition,
                                 guint8                 paint_opacity)
 {
   StThemeNodeTransitionPrivate *priv = transition->priv;
+  CoglFramebuffer *fb = cogl_get_draw_framebuffer ();
 
   CoglColor constant;
   float tex_coords[] = {
@@ -347,20 +359,20 @@ st_theme_node_transition_paint (StThemeNodeTransition *transition,
         return;
     }
 
-  cogl_color_set_from_4f (&constant, 0., 0., 0.,
-                          clutter_timeline_get_progress (priv->timeline));
-  cogl_material_set_layer_combine_constant (priv->material, 1, &constant);
+  cogl_color_init_from_4f (&constant, 0., 0., 0.,
+                           clutter_timeline_get_progress (priv->timeline));
+  cogl_pipeline_set_layer_combine_constant (priv->material, 1, &constant);
 
-  cogl_material_set_color4ub (priv->material,
+  cogl_pipeline_set_color4ub (priv->material,
                               paint_opacity, paint_opacity,
                               paint_opacity, paint_opacity);
 
-  cogl_set_source (priv->material);
-  cogl_rectangle_with_multitexture_coords (priv->offscreen_box.x1,
-                                           priv->offscreen_box.y1,
-                                           priv->offscreen_box.x2,
-                                           priv->offscreen_box.y2,
-                                           tex_coords, 8);
+  cogl_framebuffer_draw_multitextured_rectangle (fb, priv->material,
+                                                 priv->offscreen_box.x1,
+                                                 priv->offscreen_box.y1,
+                                                 priv->offscreen_box.x2,
+                                                 priv->offscreen_box.y2,
+                                                 tex_coords, 8);
 }
 
 static void
@@ -435,7 +447,7 @@ st_theme_node_transition_dispose (GObject *object)
 static void
 st_theme_node_transition_init (StThemeNodeTransition *transition)
 {
-  transition->priv = ST_THEME_NODE_TRANSITION_GET_PRIVATE (transition);
+  transition->priv = st_theme_node_transition_get_instance_private (transition);
 
   transition->priv->old_theme_node = NULL;
   transition->priv->new_theme_node = NULL;
@@ -457,23 +469,19 @@ st_theme_node_transition_class_init (StThemeNodeTransitionClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (StThemeNodeTransitionPrivate));
-
   object_class->dispose = st_theme_node_transition_dispose;
 
   signals[COMPLETED] =
     g_signal_new ("completed",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (StThemeNodeTransitionClass, completed),
-                  NULL, NULL, NULL,
+                  0, NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   signals[NEW_FRAME] =
     g_signal_new ("new-frame",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (StThemeNodeTransitionClass, new_frame),
-                  NULL, NULL, NULL,
+                  0, NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 }
